@@ -6,6 +6,8 @@ the `astropy.coordinates` module.
 """
 from __future__ import absolute_import, division
 
+import warnings
+
 import numpy as np
 
 from astropy import units as u
@@ -14,12 +16,12 @@ from astropy.coordinates.representation import (CartesianRepresentation,
                                                 SphericalRepresentation)
 from astropy.coordinates.baseframe import (BaseCoordinateFrame,
                                            RepresentationMapping)
-from astropy.coordinates import FrameAttribute
+from astropy.coordinates import Attribute, CoordinateAttribute, ConvertError
 
-from sunpy import sun  # For Carrington rotation number
+from sunpy import sun
 from .representation import (SphericalWrap180Representation, UnitSphericalWrap180Representation)
 
-from .frameattributes import TimeFrameAttributeSunPy
+from .frameattributes import TimeFrameAttributeSunPy, ObserverCoordinateAttribute
 
 RSUN_METERS = sun.constants.get('radius').si.to(u.m)
 DSUN_METERS = sun.constants.get('mean distance').si.to(u.m)
@@ -49,6 +51,9 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
     radius: `astropy.units.Quantity` object.
         This quantity holds the radial distance. If not specified, it is, by
         default, the radius of the photosphere. Optional.
+    obstime: SunPy Time
+        The date and time of the observation, used to convert to heliographic
+        carrington coordinates.
 
     Examples
     --------
@@ -57,16 +62,16 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
     >>> import astropy.units as u
     >>> sc = SkyCoord(1*u.deg, 1*u.deg, 2*u.km,
     ...               frame="heliographic_stonyhurst",
-    ...               dateobs="2010/01/01T00:00:45")
+    ...               obstime="2010/01/01T00:00:45")
     >>> sc
-    <SkyCoord (HelioGraphicStonyhurst): dateobs=2010-01-01 00:00:45,
+    <SkyCoord (HeliographicStonyhurst): obstime=2010-01-01 00:00:45,
     lon=1.0 deg, lat=1.0 deg, rad=2.0 km>
     >>> sc.frame
-    <HelioGraphicStonyhurst Coordinate: dateobs=2010-01-01 00:00:45,
+    <HeliographicStonyhurst Coordinate: obstime=2010-01-01 00:00:45,
     lon=1.0 deg, lat=1.0 deg, rad=2.0 km>
-    >>> sc = SkyCoord(HelioGraphicStonyhurst(-10*u.deg, 2*u.deg))
+    >>> sc = SkyCoord(HeliographicStonyhurst(-10*u.deg, 2*u.deg))
     >>> sc
-    <SkyCoord (HelioGraphicStonyhurst): dateobs=None, lon=-10.0 deg,
+    <SkyCoord (HeliographicStonyhurst): obstime=None, lon=-10.0 deg,
     lat=2.0 deg, rad=695508.0 km>
 
     Notes
@@ -90,7 +95,7 @@ class HeliographicStonyhurst(BaseCoordinateFrame):
         ]
     }
 
-    dateobs = TimeFrameAttributeSunPy()
+    obstime = TimeFrameAttributeSunPy()
 
     def __init__(self, *args, **kwargs):
         _rep_kwarg = kwargs.get('representation', None)
@@ -136,6 +141,9 @@ class HeliographicCarrington(HeliographicStonyhurst):
     radius: `astropy.units.Quantity` object, optional, must be keyword.
         This quantity holds the radial distance. If not specified, it is, by
         default, the solar radius. Optional, must be keyword.
+    obstime: SunPy Time
+        The date and time of the observation, used to convert to heliographic
+        carrington coordinates.
 
     Examples
     --------
@@ -144,14 +152,14 @@ class HeliographicCarrington(HeliographicStonyhurst):
     >>> import astropy.units as u
     >>> sc = SkyCoord(1*u.deg, 2*u.deg, 3*u.km,
     ...               frame="heliographic_carrington",
-    ...               dateobs="2010/01/01T00:00:30")
+    ...               obstime="2010/01/01T00:00:30")
     >>> sc
-    <SkyCoord (HelioGraphicCarrington): dateobs=2010-01-01 00:00:30,
+    <SkyCoord (HelioGraphicCarrington): obstime=2010-01-01 00:00:30,
     lon=1.0 deg, lat=2.0 deg, rad=3.0 km>
     >>> sc = SkyCoord([1,2,3]*u.deg, [4,5,6]*u.deg, [5,6,7]*u.km,
-    dateobs="2010/01/01T00:00:45", frame="heliographic_carrington")
+    obstime="2010/01/01T00:00:45", frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HelioGraphicCarrington): dateobs=2010-01-01 00:00:45,
+    <SkyCoord (HelioGraphicCarrington): obstime=2010-01-01 00:00:45,
     (lon, lat, rad) in (deg, deg, km)
         [(1.0, 4.0, 5.0), (2.0, 5.0, 6.0), (3.0, 6.0, 7.0)]>
     """
@@ -172,7 +180,7 @@ class HeliographicCarrington(HeliographicStonyhurst):
         ]
     }
 
-    dateobs = TimeFrameAttributeSunPy()
+    obstime = TimeFrameAttributeSunPy()
 
 
 class Heliocentric(BaseCoordinateFrame):
@@ -196,9 +204,11 @@ class Heliocentric(BaseCoordinateFrame):
         Y-axis coordinate, optional, must be keyword.
     z: `Quantity` object. Shared by both representations.
         Z-axis coordinate, optional, must be keyword.
-    D0: `Quantity` object.
-        Represents the distance between the observer and the Sun center.
-        Defaults to 1AU.
+    observer: `~sunpy.coordinates.frames.HeliographicStonyhurst`
+        The coordinate of the observer in the solar system.
+    obstime: SunPy Time
+        The date and time of the observation, used to convert to heliographic
+        carrington coordinates.
 
     Examples
     --------
@@ -206,14 +216,14 @@ class Heliocentric(BaseCoordinateFrame):
     >>> import sunpy.coordinates
     >>> import astropy.units as u
     >>> sc = SkyCoord(CartesianRepresentation(10*u.km, 1*u.km, 2*u.km),
-    ...               dateobs="2011/01/05T00:00:50", frame="heliocentric")
+    ...               obstime="2011/01/05T00:00:50", frame="heliocentric")
     >>> sc
-    <SkyCoord (HelioCentric): dateobs=2011-01-05 00:00:50, D0=149597870.7 km,
+    <SkyCoord (HelioCentric): obstime=2011-01-05 00:00:50, D0=149597870.7 km,
     x=10.0 km, y=1.0 km, z=2.0 km>
     >>> sc = SkyCoord([1,2]*u.km, [3,4]*u.m, [5,6]*u.cm, frame="heliocentric",
-    dateobs="2011/01/01T00:00:54")
+    obstime="2011/01/01T00:00:54")
     >>> sc
-    <SkyCoord (HelioCentric): dateobs=2011-01-01 00:00:54, D0=149597870.7 km,
+    <SkyCoord (HelioCentric): obstime=2011-01-01 00:00:54, D0=149597870.7 km,
     (x, y, z) in (km, m, cm)
         [(1.0, 3.0, 5.0), (2.0, 4.0, 6.0)]>
     """
@@ -224,11 +234,8 @@ class Heliocentric(BaseCoordinateFrame):
         'cylindrical': [RepresentationMapping('phi', 'psi', u.deg)]
     }
 
-    # d = FrameAttribute(default=(1*u.au).to(u.km))
-    D0 = FrameAttribute(default=(1*u.au).to(u.km))
-    dateobs = TimeFrameAttributeSunPy()
-    L0 = FrameAttribute(default=0*u.deg)
-    B0 = FrameAttribute(default=0*u.deg)
+    obstime = TimeFrameAttributeSunPy()
+    observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
 
 
 class Helioprojective(BaseCoordinateFrame):
@@ -250,28 +257,30 @@ class Helioprojective(BaseCoordinateFrame):
         Y-axis coordinate.
     distance: `~astropy.units.Quantity`
         The radial distance from the observer to the coordinate point.
-    L0: `~astropy.coordinates.Angle`
-        The Heliographic (Stonyhurst) Longitude of the observer.
-    B0: `~astropy.coordinates.Angle` or `~astropy.units.Quantity`
-        The Heliographic (Stonyhurst) Latitude of the observer.
-    D0: `Quantity` object.
-        Represents the distance between observer and solar center.
-        **Defaults to 1 AU**.
+    obstime: SunPy Time
+        The date and time of the observation, used to convert to heliographic
+        carrington coordinates.
+    observer: `~sunpy.coordinates.frames.HeliographicStonyhurst`
+        The coordinate of the observer in the solar system.
+    rsun: `~astropy.units.Quantity`
+        The physical (length) radius of the Sun. Used to calculate the position
+        of the limb for calculating distance from the observer to the
+        coordinate.
 
     Examples
     --------
     >>> from astropy.coordinates import SkyCoord
     >>> import sunpy.coordinates
     >>> import astropy.units as u
-    >>> sc = SkyCoord(0*u.deg, 0*u.deg, 5*u.km, dateobs="2010/01/01T00:00:00",
+    >>> sc = SkyCoord(0*u.deg, 0*u.deg, 5*u.km, obstime="2010/01/01T00:00:00",
     ...               frame="helioprojective")
     >>> sc
-    <SkyCoord (HelioProjective): dateobs=2010-01-01 00:00:00, D0=149597870.7 km
+    <SkyCoord (HelioProjective): obstime=2010-01-01 00:00:00, D0=149597870.7 km
     , Tx=0.0 arcsec, Ty=0.0 arcsec, distance=5.0 km>
-    >>> sc = SkyCoord(0*u.deg, 0*u.deg, dateobs="2010/01/01T00:00:00",
+    >>> sc = SkyCoord(0*u.deg, 0*u.deg, obstime="2010/01/01T00:00:00",
     frame="helioprojective")
     >>> sc
-    <SkyCoord (HelioProjective): dateobs=2010-01-01 00:00:00, D0=149597870.7 km
+    <SkyCoord (HelioProjective): obstime=2010-01-01 00:00:00, D0=149597870.7 km
     , Tx=0.0 arcsec, Ty=0.0 arcsec, distance=149597870.7 km>
     """
 
@@ -298,11 +307,9 @@ class Helioprojective(BaseCoordinateFrame):
         ]
     }
 
-    D0 = FrameAttribute(default=(1*u.au).to(u.km))
-    dateobs = TimeFrameAttributeSunPy()
-    L0 = FrameAttribute(default=0*u.deg)
-    B0 = FrameAttribute(default=0*u.deg)
-    rsun = FrameAttribute(default=RSUN_METERS.to(u.km))
+    obstime = TimeFrameAttributeSunPy()
+    rsun = Attribute(default=RSUN_METERS.to(u.km))
+    observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
 
     def __init__(self, *args, **kwargs):
         _rep_kwarg = kwargs.get('representation', None)
@@ -326,7 +333,7 @@ class Helioprojective(BaseCoordinateFrame):
 
     def calculate_distance(self):
         """
-        This method calculates the third coordnate of the Helioprojective
+        This method calculates the third coordinate of the Helioprojective
         frame. It assumes that the coordinate point is on the disk of the Sun
         at the rsun radius.
 
@@ -342,12 +349,18 @@ class Helioprojective(BaseCoordinateFrame):
         if isinstance(self._data, SphericalRepresentation):
             return self
 
+        if not isinstance(self.observer, BaseCoordinateFrame):
+            raise ConvertError("Cannot calculate distance to the solar disk "
+                               "for observer '{}' "
+                               "without `obstime` being specified.".format(self.observer))
+
         rep = self.represent_as(UnitSphericalWrap180Representation)
         lat, lon = rep.lat, rep.lon
         alpha = np.arccos(np.cos(lat) * np.cos(lon)).to(lat.unit)
-        c = self.D0**2 - self.rsun**2
-        b = -2 * self.D0.to(u.m) * np.cos(alpha)
+        c = self.observer.radius**2 - self.rsun**2
+        b = -2 * self.observer.radius * np.cos(alpha)
         d = ((-1*b) - np.sqrt(b**2 - 4*c)) / 2
+
         return self.realize_frame(SphericalWrap180Representation(lon=lon,
                                                                  lat=lat,
                                                                  distance=d))
